package com.engine.organization.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.engine.core.impl.Service;
import com.engine.organization.entity.DeleteParam;
import com.engine.organization.entity.extend.bo.ExtendGroupBO;
import com.engine.organization.entity.extend.bo.ExtendInfoBO;
import com.engine.organization.entity.extend.param.ExtendInfoFieldParam;
import com.engine.organization.entity.extend.param.ExtendTitleSaveParam;
import com.engine.organization.entity.extend.po.ExtendGroupPO;
import com.engine.organization.entity.extend.po.ExtendInfoPO;
import com.engine.organization.entity.extend.po.ExtendTitlePO;
import com.engine.organization.entity.fieldset.param.FieldTypeTreeParam;
import com.engine.organization.entity.fieldset.vo.TypeTreeVO;
import com.engine.organization.enums.DeleteTypeEnum;
import com.engine.organization.enums.ModuleTypeEnum;
import com.engine.organization.mapper.extend.ExtendGroupMapper;
import com.engine.organization.mapper.extend.ExtendInfoMapper;
import com.engine.organization.mapper.extend.ExtendTitleMapper;
import com.engine.organization.service.FieldDefinedService;
import com.engine.organization.util.OrganizationAssert;
import com.engine.organization.util.db.MapperProxyFactory;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Author weaver_cl
 * @Description:
 * @Date 2022/6/13
 * @Version V1.0
 **/
public class FieldDefinedServiceImpl extends Service implements FieldDefinedService {

    private ExtendGroupMapper getExtendGroupMapper() {
        return MapperProxyFactory.getProxy(ExtendGroupMapper.class);
    }

    private ExtendTitleMapper getExtendTitleMapper() {
        return MapperProxyFactory.getProxy(ExtendTitleMapper.class);
    }

    private ExtendInfoMapper getExtendInfoMapper() {
        return MapperProxyFactory.getProxy(ExtendInfoMapper.class);
    }

    @Override
    public List<TypeTreeVO> getTree(ModuleTypeEnum moduleTypeEnum) {
        Integer extendType = moduleTypeEnum.getValue();
        List<ExtendGroupPO> extendGroupPOS = MapperProxyFactory.getProxy(ExtendGroupMapper.class).listByType(extendType);
        return ExtendGroupBO.buildTypeTree(extendGroupPOS);
    }

    @Override
    public Map<String, Object> getTabInfo(String groupType) {
        OrganizationAssert.notBlank(groupType, "分组信息有误，请确认");
        Map<String, Object> datas = new HashMap<>();
        Long groupId = Long.parseLong(groupType);
        List<ExtendTitlePO> extendTitleList = getExtendTitleMapper().getTitlesByGroupID(groupId);
        List<Map<String, Object>> tabs = new ArrayList<>();
        int idx = 1;
        for (ExtendTitlePO extendTitle : extendTitleList) {
            Map<String, Object> tab = new HashMap<>();
            tab.put("tabkey", idx++);
            tab.put("title", extendTitle.getTitle());
            tab.put("isShow", extendTitle.getIsShow());
            tab.put("groupid", extendTitle.getId());
            tab.put("viewAttr", extendTitle.getGroupId() < 0 ? 1 : 2);
            tab.put("editable", getExtendInfoMapper().countFieldsByGroupId(extendTitle.getId()) == 0);
            tabs.add(tab);
        }
        datas.put("status", "1");
        datas.put("tabs", tabs);
        return datas;
    }

    @Override
    public int saveFields(String data) {
        if (StringUtils.isEmpty(data)) {
            return 0;
        }
        JSONObject jsonObject = JSON.parseObject(data);
        // extend_group主键
        Long groupType = jsonObject.getLong("groupType");
        ExtendGroupPO extendGroup = getExtendGroupMapper().getGroupById(groupType);

        // 主表、主表拓展表为extend_title主键；明细表为extend_group主键
        Long extendGroupId = jsonObject.getLong("groupId");

        JSONArray records = (JSONArray) jsonObject.get("records");
        for (int i = 0; i < records.size(); i++) {
            // 存在ID则更新 ，不存在ID则增加
            ExtendInfoFieldParam infoFieldParam = JSONObject.toJavaObject((JSONObject) records.get(i), ExtendInfoFieldParam.class);
            // TODO
            ExtendInfoPO extendInfo = ExtendInfoBO.convertFieldParam2ExtendInfo(user, infoFieldParam, extendGroup, extendGroupId, i + 1);

            if (null != extendInfo.getId()) {
                // 更新 extend_title数据
                getExtendInfoMapper().updateExtendInfo(extendInfo);

            } else {
                // 新增extend_title数据

                // 新增表结构
            }
        }
        return 0;
    }

    @Override
    public Map<String, Object> saveGroup(ExtendTitleSaveParam param) {
        Map<String, Object> datas = new HashMap<>();
        Long groupType = param.getGroupType();
        Set<Long> lsDelGroupId = new HashSet<>();
        List<ExtendTitlePO> extendTitleList = getExtendTitleMapper().getTitlesByGroupID(groupType);
        extendTitleList.forEach(item -> lsDelGroupId.add(item.getId()));
        String data = param.getData();
        JSONObject jsonObject = JSON.parseObject(data);
        Set<Long> ids = new HashSet<>();
        JSONArray records = (JSONArray) jsonObject.get("records");
        for (int i = 0; i < records.size(); i++) {
            ExtendTitlePO extendTitle = new ExtendTitlePO();
            JSONObject object = (JSONObject) records.get(i);
            extendTitle.setId(object.getLong("id"));
            extendTitle.setGroupId(groupType);
            extendTitle.setIsShow(object.getString("isShow"));
            extendTitle.setTitle(object.getString("groupName"));
            extendTitle.setShowOrder(i + 1);

            if (null != extendTitle.getId()) {
                lsDelGroupId.remove(extendTitle.getId());
            }

            if (null == extendTitle.getId()) {
                extendTitle.setCreator((long) user.getUID());
                extendTitle.setCreateTime(new Date());
                extendTitle.setDeleteType(0);
                getExtendTitleMapper().insertIgnoreNull(extendTitle);
            } else {
                extendTitle.setUpdateTime(new Date());
                getExtendTitleMapper().updateExtendTitle(extendTitle);
            }
            ids.add(extendTitle.getId());
        }
        if (CollectionUtils.isNotEmpty(lsDelGroupId)) {
            getExtendTitleMapper().deleteExtendTitleByIds(lsDelGroupId);
        }
        datas.put("groupid", StringUtils.join(ids, ","));
        datas.put("status", "1");
        return datas;
    }

    @Override
    public Map<String, Object> saveTitle(ExtendTitleSaveParam param) {
        Map<String, Object> datas = new HashMap<>();

        List<ExtendTitlePO> titlesByGroupID = getExtendTitleMapper().getTitlesByGroupID(param.getGroupType());
        ExtendTitlePO extendTitlePO = titlesByGroupID.stream().max(Comparator.comparing(ExtendTitlePO::getShowOrder)).get();
        Set<Long> ids = titlesByGroupID.stream().map(ExtendTitlePO::getId).collect(Collectors.toSet());
        ExtendTitlePO extendTitle = new ExtendTitlePO();
        extendTitle.setGroupId(param.getGroupType());
        extendTitle.setIsShow(param.getIsShow());
        extendTitle.setTitle(param.getGroupName());
        extendTitle.setShowOrder(null == extendTitlePO ? 0 : extendTitlePO.getShowOrder() + 1);
        extendTitle.setCreator((long) user.getUID());
        extendTitle.setCreateTime(new Date());
        extendTitle.setDeleteType(0);
        getExtendTitleMapper().insertIgnoreNull(extendTitle);
        ids.add(extendTitle.getId());

        datas.put("groupid", StringUtils.join(ids, ","));
        datas.put("status", "1");
        return datas;
    }

    @Override
    public void changeTree(ModuleTypeEnum moduleTypeEnum, FieldTypeTreeParam fieldTypeTreeParam) {
        if (fieldTypeTreeParam.getId() == null) {
            List<ExtendGroupPO> extendGroupPOList = MapperProxyFactory.getProxy(ExtendGroupMapper.class).listGroupByPid(fieldTypeTreeParam.getParentId());
            ExtendGroupPO extendGroup = extendGroupPOList.stream().max(Comparator.comparing(ExtendGroupPO::getShowOrder)).get();
            ExtendGroupPO extendGroupPO = buildExtendGroupType(moduleTypeEnum, fieldTypeTreeParam,extendGroup.getShowOrder());
            MapperProxyFactory.getProxy(ExtendGroupMapper.class).insertIgnoreNull(extendGroupPO);
            //默认新增title
            List<ExtendTitlePO> titlesByGroupID = getExtendTitleMapper().getTitlesByGroupID(extendGroup.getId());
            ExtendTitlePO extendTitle = titlesByGroupID.stream().max(Comparator.comparing(ExtendTitlePO::getShowOrder)).get();
            ExtendTitlePO extendTitlePO = buildExtendTitleType(extendGroupPO,extendTitle.getShowOrder());
            MapperProxyFactory.getProxy(ExtendTitleMapper.class).insertIgnoreNull(extendTitlePO);
        } else {
            MapperProxyFactory.getProxy(ExtendGroupMapper.class).update(fieldTypeTreeParam.getId(), fieldTypeTreeParam.getName());
        }

    }

    @Override
    public int deleteTitle(Long id) {
        return getExtendTitleMapper().deleteExtendTitleByIds(DeleteParam.builder().ids(id.toString()).build().getIds());
    }

    @Override
    public Map<String, Object> getFieldDefinedInfo(ExtendTitleSaveParam param) {

        return null;
    }

    @Override
    public void deleteTree(Long id) {
        MapperProxyFactory.getProxy(ExtendGroupMapper.class).delete(id);
    }

    @Override
    public void saveTree(ModuleTypeEnum moduleTypeEnum, FieldTypeTreeParam fieldTypeTreeParam) {
        List<Long> collect = MapperProxyFactory.getProxy(ExtendGroupMapper.class)
                .listGroupByPid(fieldTypeTreeParam.getParentId()).stream().map(ExtendGroupPO::getId).collect(Collectors.toList());
        Integer extendType = moduleTypeEnum.getValue();
        List<ExtendGroupPO> extendGroupPOS = JSONArray.parseArray(fieldTypeTreeParam.getData(),ExtendGroupPO.class);

        List<Long> collect1 = extendGroupPOS.stream().map(ExtendGroupPO::getId).collect(Collectors.toList());
        Set<Long> removeSet = new HashSet<>();
        collect1.forEach(e->{
            if(!collect.contains(e)){
                removeSet.add(e);
            }
        });
        MapperProxyFactory.getProxy(ExtendGroupMapper.class).batchDelete(removeSet);

        int showOrder = 0;
        extendGroupPOS.forEach(e -> {
            e.setExtendType(extendType);
            e.setPid(fieldTypeTreeParam.getParentId());
            e.setIsShow(fieldTypeTreeParam.getIsShow());
            e.setShowOrder(showOrder+1);
            e.setCreator((long)user.getUID());
            e.setDeleteType(DeleteTypeEnum.NOT_DELETED.getValue());
            e.setCreateTime(new Date());
            e.setUpdateTime(new Date());

            if (e.getId() == null){//新增
                MapperProxyFactory.getProxy(ExtendGroupMapper.class).insertIgnoreNull(e);
            }else { //更新
                MapperProxyFactory.getProxy(ExtendGroupMapper.class).updateNameAndOrder(e.getId(),e.getGroupName(),e.getShowOrder());
            }
        });




    }

    private ExtendTitlePO buildExtendTitleType(ExtendGroupPO extendGroupPO,Integer max) {
        return ExtendTitlePO.builder()
                .groupId(extendGroupPO.getId())
                .title(extendGroupPO.getGroupName())
                .showOrder(++max)
                .isShow("1")
                .creator((long) user.getUID())
                .deleteType(DeleteTypeEnum.NOT_DELETED.getValue())
                .createTime(new Date())
                .updateTime(new Date()).build();
    }


    private ExtendGroupPO buildExtendGroupType(ModuleTypeEnum moduleTypeEnum, FieldTypeTreeParam fieldTypeTreeParam,Integer max) {
        return ExtendGroupPO.builder()
                .extendType(moduleTypeEnum.getValue())
                .groupName(fieldTypeTreeParam.getName())
                .showOrder(++max)
                .creator((long) user.getUID())
                .deleteType(DeleteTypeEnum.NOT_DELETED.getValue())
                .createTime(new Date())
                .updateTime(new Date())
                .pid(fieldTypeTreeParam.getParentId())
                .isShow(fieldTypeTreeParam.getIsShow() == null ? "1" : fieldTypeTreeParam.getIsShow()).build();
    }


}
